//-----------------------------------------------------------------------------
// Copyright Notice
//
//   Copyright 2002 Sandia Corporation. Under the terms
//   of Contract DE-AC04-94AL85000 with Sandia Corporation, the U.S.
//   Government retains certain rights in this software.
//
//    Xyce(TM) Parallel Electrical Simulator
//    Copyright (C) 2002-2016 Sandia Corporation
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------

//-------------------------------------------------------------------------
//
// Purpose        :
//
// Special Notes  :
//
// Creator        : Eric Keiter
//
// Creation Date  : 5/22/00
//
//
//
//
//-------------------------------------------------------------------------

#ifndef Xyce_N_UTL_NetlistLocation_h
#define Xyce_N_UTL_NetlistLocation_h

#include <iosfwd>
#include <string>

#include <N_UTL_Marshal.h>

namespace Xyce {

struct NetlistLocation
{
  NetlistLocation()
    : filename_(),
      lineNumber_(0)
  {}

  NetlistLocation(const std::string &filename, const int line_number)
    : filename_(filename),
      lineNumber_(line_number)
  {}

  NetlistLocation(const NetlistLocation &netlist_location)
    : filename_(netlist_location.filename_),
      lineNumber_(netlist_location.lineNumber_)
  {}

  NetlistLocation &operator=(const NetlistLocation &netlist_location) 
  {
    filename_ = netlist_location.filename_;
    lineNumber_ = netlist_location.lineNumber_;

    return *this;
  }

  NetlistLocation &setFilename(const std::string &filename) 
  {
    filename_ = filename;
    return *this;
  }

  const std::string &getFilename() const 
  {
    return filename_;
  }

  NetlistLocation &setLineNumber(int line_number) 
  {
    lineNumber_ = line_number;
    return *this;
  }

  int getLineNumber() const 
  {
    return lineNumber_;
  }

private:
  std::string   filename_;
  int           lineNumber_;
};

bool operator<(const NetlistLocation &s0, const NetlistLocation &s1);

std::ostream &operator<<(std::ostream &os, const NetlistLocation &netlist_locationc);

Util::Marshal &operator<<(Util::Marshal &mout, const NetlistLocation &netlist_location);

Util::Marshal &operator>>(Util::Marshal &min, NetlistLocation &netlist_location);

} // namespace Xyce

#endif // Xyce_N_UTL_NetlistLocation_h

